'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _findMatchingVNodes = require('./lib/findMatchingVNodes');

var _findMatchingVNodes2 = _interopRequireDefault(_findMatchingVNodes);

var _vm = require('./lib/vm');

var _VueWrapper = require('./VueWrapper');

var _VueWrapper2 = _interopRequireDefault(_VueWrapper);

var _isDomSelector = require('./lib/isDomSelector');

var _isDomSelector2 = _interopRequireDefault(_isDomSelector);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function isValidSelector(selector) {
  if ((0, _isDomSelector2.default)(selector)) {
    return true;
  }
  if (typeof selector === 'function') {
    return false;
  }

  if (selector === null) {
    return false;
  }

  if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) !== 'object') {
    return false;
  }

  if (typeof selector.name !== 'string') {
    return false;
  }

  return true;
}

var Wrapper = function () {
  function Wrapper(vNode, update) {
    _classCallCheck(this, Wrapper);

    this.vNode = vNode;
    this.element = vNode.elm;
    this.update = update;
  }

  _createClass(Wrapper, [{
    key: 'computed',
    value: function computed() {
      if (!this.isVueComponent) {
        throw new Error('wrapper.computed() can only be called on a Vue instance');
      }

      return this.vm.$options.computed;
    }
  }, {
    key: 'contains',
    value: function contains(selector) {
      if (!isValidSelector(selector)) {
        throw new Error('wrapper.find() must be passed a valid CSS selector or a Vue constructor');
      }

      if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === 'object') {
        var vm = this.vm || this.vNode.context.$root;
        return (0, _vm.findVueComponents)(vm, selector.name).length > 0;
      }

      return this.element.querySelectorAll(selector).length > 0;
    }
  }, {
    key: 'data',
    value: function data() {
      if (!this.isVueComponent) {
        throw new Error('wrapper.data() can only be called on a Vue instance');
      }

      return this.vm._data;
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      if (!this.isVueComponent) {
        throw new Error('wrapper.destroy() can only be called on a Vue instance');
      }
      this.vm.$destroy();
    }
  }, {
    key: 'find',
    value: function find(selector) {
      var _this = this;

      if (!isValidSelector(selector)) {
        throw new Error('wrapper.find() must be passed a valid CSS selector or a Vue constructor');
      }

      if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === 'object') {
        var vm = this.vm || this.vNode.context.$root;
        var components = (0, _vm.findVueComponents)(vm, selector.name);
        return components.map(function (component) {
          return new _VueWrapper2.default(component);
        });
      }

      var nodes = (0, _findMatchingVNodes2.default)(this.vNode, selector);

      return nodes.map(function (node) {
        return new Wrapper(node, _this.update);
      });
    }
  }, {
    key: 'hasAttribute',
    value: function hasAttribute(attribute, value) {
      if (typeof attribute !== 'string') {
        throw new Error('wrapper.hasAttribute() must be passed attribute as a string');
      }

      if (typeof value !== 'string') {
        throw new Error('wrapper.hasAttribute() must be passed value as a string');
      }

      return this.element.getAttribute(attribute) === value;
    }
  }, {
    key: 'hasClass',
    value: function hasClass(className) {
      if (typeof className !== 'string') {
        throw new Error('wrapper.hasClass() must be passed a string');
      }

      return this.element.className.split(' ').indexOf(className) !== -1;
    }
  }, {
    key: 'hasStyle',
    value: function hasStyle(style, value) {
      if (typeof style !== 'string') {
        throw new Error('wrapper.hasStyle() must be passed style as a string');
      }

      if (typeof value !== 'string') {
        throw new Error('wrapper.hasClass() must be passed value as string');
      }

      if (navigator.userAgent.includes && navigator.userAgent.includes('node.js')) {
        console.warn('wrapper.hasStyle is not fully supported when running jsdom - only inline styles are supported');
      }

      var body = document.querySelector('body');
      var node = body.insertBefore(this.element, null);
      var mockDiv = document.createElement('div');
      var mockNode = body.insertBefore(mockDiv, null);

      mockDiv.style[style] = value;

      return window.getComputedStyle(node)[style] === window.getComputedStyle(mockNode)[style];
    }
  }, {
    key: 'html',
    value: function html() {
      var tmp = document.createElement('div');
      tmp.appendChild(this.element);
      return tmp.innerHTML;
    }
  }, {
    key: 'is',
    value: function is(selector) {
      if (!isValidSelector(selector)) {
        throw new Error('wrapper.is() must be passed a valid CSS selector or a Vue constructor');
      }

      if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === 'object') {
        if (!this.isVueComponent) {
          return false;
        }

        return (0, _vm.vmCtorMatchesName)(this.vm, selector.name);
      }

      if (selector[0] === '.') {
        return this.element.className.split(' ').indexOf(selector.substr(1)) !== -1;
      }

      if (selector[0] === '#') {
        return this.element.getAttribute('id') === selector.substr(1);
      }

      return this.element.tagName === selector.toUpperCase();
    }
  }, {
    key: 'isEmpty',
    value: function isEmpty() {
      return this.vNode.children === undefined;
    }
  }, {
    key: 'methods',
    value: function methods() {
      if (!this.isVueComponent) {
        throw new Error('wrapper.methods() can only be called on a Vue instance');
      }

      return this.vm.$options.methods;
    }
  }, {
    key: 'name',
    value: function name() {
      if (this.isVueComponent) {
        return this.vm.$options.name;
      }

      return this.vNode.tag;
    }
  }, {
    key: 'propsData',
    value: function propsData() {
      if (!this.isVueComponent) {
        throw new Error('wrapper.propsData() can only be called on a Vue instance');
      }

      return this.vm.$options.propsData;
    }
  }, {
    key: 'setData',
    value: function setData(data) {
      var _this2 = this;

      if (!this.isVueComponent) {
        throw new Error('wrapper.setData() can only be called on a Vue instance');
      }

      Object.keys(data).forEach(function (key) {
        _this2.vm._data[key] = data[key];
      });
      this.update();
    }
  }, {
    key: 'simulate',
    value: function simulate(type) {
      if (typeof type !== 'string') {
        throw new Error('wrapper.simulate() must be passed a string');
      }

      var eventObject = new window.Event(type);
      this.element.dispatchEvent(eventObject);
      this.update();
    }
  }, {
    key: 'style',
    value: function style() {
      console.warn('wrapper.style() is deprecated and will be removed from future versions. Use wrapper.hasStyle() instead');
      var node = document.querySelector('body').insertBefore(this.element, null);
      return window.getComputedStyle(node);
    }
  }, {
    key: 'text',
    value: function text() {
      return this.element.textContent;
    }
  }]);

  return Wrapper;
}();

exports.default = Wrapper;